/*
Lua 5.2 & Luna Five: Account Example
Date: 16 May 2013
Website: http://blog.p86.dk
*/

#include "account.h"

#define method(class, name) {#name, &class::name}

//Name to use to address the class in Lua
const char Account::className[] = "Account";

//List of class properties that one can set/get from Lua
const Luna<Account>::PropertyType Account::properties[] = {
  {0,0}
};

//List of class methods to make available in Lua
const Luna<Account>::FunctionType Account::methods[] = {
  method(Account, Deposit),
  method(Account, Withdraw),
  method(Account, Balance),
  {0,0}
};

//==========================================================

/**
Constructor
@param L The Lua state
*/
Account::Account(lua_State *L) { 
  m_balance = lua_tointeger(L, 1);
}

/**
Destructor
*/
Account::~Account() { 
  printf("C++: Deleted Account (%p)\n", this); 
}

/**
Deposits an amount into the account
@param L The Lua state
@return The number of returned variables on the stack. None in this case.
*/
int Account::Deposit(lua_State *L) { 
  m_balance += lua_tointeger(L, 1);  
  return 0; 
}

/**
Withdraws an amount from the account
@param L The Lua state
@return The number of returned variables on the stack. None in this case.
*/
int Account::Withdraw(lua_State *L) { 
  m_balance -= lua_tointeger(L, 1); 
  return 0; 
}

/**
Accessor Method
Get the balance for the account
@param L The Lua state
@return The number of returned variables on the stack. The balance in this case.
*/
int Account::Balance(lua_State *L) { 
  lua_pushnumber(L, m_balance); 
  return 1; 
}

